@@ -1,11 +1,18 @@
 #include "lc_global.h"
 #include "lc_qglwidget.h"
 #include "lc_glwidget.h"
+#include "lc_glextensions.h"
 #include "project.h"
 #include "lc_library.h"
 #include "lc_application.h"
-#include "lc_qmainwindow.h"
 #include "lc_mainwindow.h"
+#include "lc_context.h"
+#include "view.h"
+#include "texfont.h"
+#include "lc_texture.h"
+#include "lc_mesh.h"
+
+static int gWidgetCount;
 
 void lcGLWidget::MakeCurrent()
 {
@@ -16,63 +23,9 @@ void lcGLWidget::MakeCurrent()
 
 void lcGLWidget::Redraw()
 {
-	QGLWidget* widget = (QGLWidget*)mWidget;
+	lcQGLWidget* Widget = (lcQGLWidget*)mWidget;
 
-	widget->updateGL();
-}
-
-void lcGLWidget::CaptureMouse()
-{
-	QGLWidget* widget = (QGLWidget*)mWidget;
-
-	widget->grabMouse();
-}
-
-void lcGLWidget::ReleaseMouse()
-{
-	QGLWidget* widget = (QGLWidget*)mWidget;
-
-	widget->releaseMouse();
-}
-
-void* lcGLWidget::GetExtensionAddress(const char* FunctionName)
-{
-	QGLWidget* Widget = (QGLWidget*)mWidget;
-
-	return (void*)Widget->context()->getProcAddress(FunctionName);
-}
-
-void lcGLWidget::ShowPopupMenu()
-{
-	QGLWidget* Widget = (QGLWidget*)mWidget;
-
-	lcQMainWindow *mainWindow = (lcQMainWindow*)gMainWindow->mHandle;
-	QAction **actions = mainWindow->actions;
-
-	QMenu *popup = new QMenu(Widget);
-
-	QMenu *tools = new QMenu("Tools");
-	popup->addMenu(tools);
-	for (int actionIdx = LC_EDIT_ACTION_FIRST; actionIdx <= LC_EDIT_ACTION_LAST; actionIdx++)
-		tools->addAction(actions[actionIdx]);
-
-	QMenu *cameras = new QMenu("Cameras");
-	popup->addMenu(cameras);
-	cameras->addAction(actions[LC_VIEW_CAMERA_NONE]);
-
-	for (int actionIdx = LC_VIEW_CAMERA_FIRST; actionIdx <= LC_VIEW_CAMERA_LAST; actionIdx++)
-		cameras->addAction(actions[actionIdx]);
-
-	cameras->addSeparator();
-	cameras->addAction(actions[LC_VIEW_CAMERA_RESET]);
-
-	popup->addSeparator();
-	popup->addAction(actions[LC_VIEW_SPLIT_HORIZONTAL]);
-	popup->addAction(actions[LC_VIEW_SPLIT_VERTICAL]);
-	popup->addAction(actions[LC_VIEW_REMOVE_VIEW]);
-	popup->addAction(actions[LC_VIEW_RESET_VIEWS]);
-
-	popup->exec(QCursor::pos());
+	Widget->mUpdateTimer.start(0);
 }
 
 void lcGLWidget::SetCursor(LC_CURSOR_TYPE CursorType)
@@ -126,11 +79,28 @@ void lcGLWidget::SetCursor(LC_CURSOR_TYPE CursorType)
 lcQGLWidget::lcQGLWidget(QWidget *parent, lcQGLWidget *share, lcGLWidget *owner, bool view)
 	: QGLWidget(parent, share)
 {
+	mWheelAccumulator = 0;
 	widget = owner;
 	widget->mWidget = this;
 
+	mUpdateTimer.setSingleShot(true);
+	connect(&mUpdateTimer, SIGNAL(timeout()), this, SLOT(updateGL()));
+
 	widget->MakeCurrent();
-	GL_InitializeSharedExtensions(widget);
+
+	// TODO: Find a better place for the grid texture and font
+	gTexFont.Load();
+	if (!gWidgetCount)
+	{
+		lcInitializeGLExtensions(context());
+		lcContext::CreateResources();
+		View::CreateResources(widget->mContext);
+
+		gPlaceholderMesh = new lcMesh;
+		gPlaceholderMesh->CreateBox();
+	}
+	gWidgetCount++;
+
 	widget->OnInitialUpdate();
 
 	preferredSize = QSize(0, 0);
@@ -146,8 +116,19 @@ lcQGLWidget::lcQGLWidget(QWidget *parent, lcQGLWidget *share, lcGLWidget *owner,
 
 lcQGLWidget::~lcQGLWidget()
 {
-	if (isView)
-		delete widget;
+	gWidgetCount--;
+	gTexFont.Release();
+	makeCurrent();
+	if (!gWidgetCount)
+	{
+		View::DestroyResources(widget->mContext);
+		lcContext::DestroyResources();
+
+		delete gPlaceholderMesh;
+		gPlaceholderMesh = NULL;
+	}
+
+	delete widget;
 }
 
 QSize lcQGLWidget::sizeHint() const
@@ -160,6 +141,12 @@ QSize lcQGLWidget::sizeHint() const
 
 void lcQGLWidget::initializeGL()
 {
+	glEnable(GL_POLYGON_OFFSET_FILL);
+	glPolygonOffset(0.5f, 0.1f);
+
+	glEnable(GL_DEPTH_TEST);
+	glDepthFunc(GL_LEQUAL);
+	glDepthMask(GL_TRUE);
 }
 
 void lcQGLWidget::resizeGL(int width, int height)
@@ -177,7 +164,7 @@ void lcQGLWidget::keyPressEvent(QKeyEvent *event)
 {
 	if (isView && event->key() == Qt::Key_Control)
 	{
-		widget->mInputState.Control = true;
+		widget->mInputState.Modifiers = event->modifiers();
 		widget->OnUpdateCursor();
 	}
 
@@ -188,7 +175,7 @@ void lcQGLWidget::keyReleaseEvent(QKeyEvent *event)
 {
 	if (isView && event->key() == Qt::Key_Control)
 	{
-		widget->mInputState.Control = false;
+		widget->mInputState.Modifiers = event->modifiers();
 		widget->OnUpdateCursor();
 	}
 
@@ -197,23 +184,36 @@ void lcQGLWidget::keyReleaseEvent(QKeyEvent *event)
 
 void lcQGLWidget::mousePressEvent(QMouseEvent *event)
 {
-	widget->mInputState.x = event->x();
-	widget->mInputState.y = widget->mHeight - event->y() - 1;
-	widget->mInputState.Control = (event->modifiers() & Qt::ControlModifier) != 0;
-	widget->mInputState.Shift = (event->modifiers() & Qt::ShiftModifier) != 0;
-	widget->mInputState.Alt = (event->modifiers() & Qt::AltModifier) != 0;
+	float scale = deviceScale();
+
+	widget->mInputState.x = event->x() * scale;
+	widget->mInputState.y = widget->mHeight - event->y() * scale - 1;
+	widget->mInputState.Modifiers = event->modifiers();
 
 	switch (event->button())
 	{
 	case Qt::LeftButton:
 		widget->OnLeftButtonDown();
 		break;
+
 	case Qt::MidButton:
 		widget->OnMiddleButtonDown();
 		break;
+
 	case Qt::RightButton:
 		widget->OnRightButtonDown();
 		break;
+
+#if (QT_VERSION >= QT_VERSION_CHECK(5, 0, 0))
+	case Qt::BackButton:
+		widget->OnBackButtonDown();
+		break;
+
+	case Qt::ForwardButton:
+		widget->OnForwardButtonDown();
+		break;
+#endif
+
 	default:
 		break;
 	}
@@ -221,23 +221,54 @@ void lcQGLWidget::mousePressEvent(QMouseEvent *event)
 
 void lcQGLWidget::mouseReleaseEvent(QMouseEvent *event)
 {
-	widget->mInputState.x = event->x();
-	widget->mInputState.y = widget->mHeight - event->y() - 1;
-	widget->mInputState.Control = (event->modifiers() & Qt::ControlModifier) != 0;
-	widget->mInputState.Shift = (event->modifiers() & Qt::ShiftModifier) != 0;
-	widget->mInputState.Alt = (event->modifiers() & Qt::AltModifier) != 0;
+	float scale = deviceScale();
+
+	widget->mInputState.x = event->x() * scale;
+	widget->mInputState.y = widget->mHeight - event->y() * scale - 1;
+	widget->mInputState.Modifiers = event->modifiers();
 
 	switch (event->button())
 	{
 	case Qt::LeftButton:
 		widget->OnLeftButtonUp();
 		break;
+
 	case Qt::MidButton:
 		widget->OnMiddleButtonUp();
 		break;
+
 	case Qt::RightButton:
 		widget->OnRightButtonUp();
 		break;
+
+#if (QT_VERSION >= QT_VERSION_CHECK(5, 0, 0))
+	case Qt::BackButton:
+		widget->OnBackButtonUp();
+		break;
+
+	case Qt::ForwardButton:
+		widget->OnForwardButtonUp();
+		break;
+#endif
+
+	default:
+		break;
+	}
+}
+
+void lcQGLWidget::mouseDoubleClickEvent(QMouseEvent *event)
+{
+	float scale = deviceScale();
+
+	widget->mInputState.x = event->x() * scale;
+	widget->mInputState.y = widget->mHeight - event->y() * scale - 1;
+	widget->mInputState.Modifiers = event->modifiers();
+
+	switch (event->button())
+	{
+	case Qt::LeftButton:
+		widget->OnLeftButtonDoubleClick();
+		break;
 	default:
 		break;
 	}
@@ -245,33 +276,41 @@ void lcQGLWidget::mouseReleaseEvent(QMouseEvent *event)
 
 void lcQGLWidget::mouseMoveEvent(QMouseEvent *event)
 {
-	widget->mInputState.x = event->x();
-	widget->mInputState.y = widget->mHeight - event->y() - 1;
-	widget->mInputState.Control = (event->modifiers() & Qt::ControlModifier) != 0;
-	widget->mInputState.Shift = (event->modifiers() & Qt::ShiftModifier) != 0;
-	widget->mInputState.Alt = (event->modifiers() & Qt::AltModifier) != 0;
+	float scale = deviceScale();
+
+	widget->mInputState.x = event->x() * scale;
+	widget->mInputState.y = widget->mHeight - event->y() * scale - 1;
+	widget->mInputState.Modifiers = event->modifiers();
 
 	widget->OnMouseMove();
 }
 
 void lcQGLWidget::wheelEvent(QWheelEvent *event)
 {
-	if (event->orientation() != Qt::Vertical)
+	if ((event->orientation() & Qt::Vertical) == 0)
 	{
 		event->ignore();
 		return;
 	}
 
-	widget->mInputState.x = event->x();
-	widget->mInputState.y = widget->mHeight - event->y() - 1;
-	widget->mInputState.Control = (event->modifiers() & Qt::ControlModifier) != 0;
-	widget->mInputState.Shift = (event->modifiers() & Qt::ShiftModifier) != 0;
-	widget->mInputState.Alt = (event->modifiers() & Qt::AltModifier) != 0;
+	float scale = deviceScale();
 
-	int numDegrees = event->delta() / 8;
-	int numSteps = numDegrees / 15;
+	widget->mInputState.x = event->x() * scale;
+	widget->mInputState.y = widget->mHeight - event->y() * scale - 1;
+	widget->mInputState.Modifiers = event->modifiers();
 
-	widget->OnMouseWheel(numSteps);
+#if (QT_VERSION >= QT_VERSION_CHECK(5, 2, 0))
+	mWheelAccumulator += event->angleDelta().y() / 8;
+#else
+	mWheelAccumulator += event->delta() / 8;
+#endif
+	int numSteps = mWheelAccumulator / 15;
+
+	if (numSteps)
+	{
+		widget->OnMouseWheel(numSteps);
+		mWheelAccumulator -= numSteps * 15;
+	}
 
 	event->accept();
 }
@@ -288,7 +327,7 @@ void lcQGLWidget::dragEnterEvent(QDragEnterEvent *event)
 
 		dataStream >> id;
 
-		lcGetActiveProject()->BeginPieceDrop(lcGetPiecesLibrary()->FindPiece(id.toLocal8Bit().data(), false));
+		((View*)widget)->BeginPieceDrag();
 	}
 	else
 		event->ignore();
@@ -299,7 +338,7 @@ void lcQGLWidget::dragLeaveEvent(QDragLeaveEvent *event)
 	if (!isView)
 		return;
 
-	lcGetActiveProject()->EndPieceDrop(false);
+	((View*)widget)->EndPieceDrag(false);
 
 	event->accept();
 }
@@ -309,7 +348,13 @@ void lcQGLWidget::dragMoveEvent(QDragMoveEvent *event)
 	if (!isView || !event->mimeData()->hasFormat("application/vnd.leocad-part"))
 		return;
 
-	lcGetActiveProject()->OnPieceDropMove(event->pos().x(), widget->mHeight - event->pos().y() - 1);
+	float scale = deviceScale();
+
+	widget->mInputState.x = event->pos().x() * scale;
+	widget->mInputState.y = widget->mHeight - event->pos().y() * scale - 1;
+	widget->mInputState.Modifiers = event->keyboardModifiers();
+
+	widget->OnMouseMove();
 
 	event->accept();
 }
@@ -319,7 +364,8 @@ void lcQGLWidget::dropEvent(QDropEvent *event)
 	if (!isView || !event->mimeData()->hasFormat("application/vnd.leocad-part"))
 		return;
 
-	lcGetActiveProject()->EndPieceDrop(true);
+	((View*)widget)->EndPieceDrag(true);
+	setFocus(Qt::MouseFocusReason);
 
 	event->accept();
 }
