@@ -4,6 +4,8 @@
 #include "lc_category.h"
 #include "lc_library.h"
 #include "pieceinf.h"
+#include "project.h"
+#include "lc_model.h"
 
 static int lcQPartsTreeSortFunc(PieceInfo* const& a, PieceInfo* const& b)
 {
@@ -25,11 +27,12 @@ static int lcQPartsTreeSortFunc(PieceInfo* const& a, PieceInfo* const& b)
 	return 0;
 }
 
-lcQPartsTree::lcQPartsTree(QWidget *parent) :
-    QTreeWidget(parent)
+lcQPartsTree::lcQPartsTree(QWidget* Parent)
+	: QTreeWidget(Parent)
 {
 	setDragEnabled(true);
 	setHeaderHidden(true);
+	setUniformRowHeights(true);
 	connect(this, SIGNAL(itemExpanded(QTreeWidgetItem*)), this, SLOT(itemExpanded(QTreeWidgetItem*)));
 
 	updateCategories();
@@ -83,39 +86,60 @@ void lcQPartsTree::updateCategories()
 		new QTreeWidgetItem(categoryItem);
 	}
 
-	searchResultsItem = new QTreeWidgetItem(this, QStringList(tr("Search Results")));
+	mModelListItem = new QTreeWidgetItem(this, QStringList(tr("Models")));
+	mSearchResultsItem = new QTreeWidgetItem(this, QStringList(tr("Search Results")));
+}
+
+void lcQPartsTree::UpdateModels()
+{
+	while (QTreeWidgetItem* Item = mModelListItem->child(0))
+		delete Item;
+
+	const lcArray<lcModel*>& Models = lcGetActiveProject()->GetModels();
+	lcModel* CurrentModel = lcGetActiveModel();
+
+	for (int ModelIdx = 0; ModelIdx < Models.GetSize(); ModelIdx++)
+	{
+		lcModel* Model = Models[ModelIdx];
+
+		if (!Model->IncludesModel(CurrentModel))
+		{
+			const lcModelProperties& Properties = Model->GetProperties();
+			QTreeWidgetItem* Item = new QTreeWidgetItem(mModelListItem, QStringList(Properties.mName));
+			Item->setData(0, PieceInfoRole, qVariantFromValue((void*)Model->GetPieceInfo()));
+			Item->setToolTip(0, Properties.mDescription);
+		}
+	}
 }
 
 void lcQPartsTree::searchParts(const QString& searchString)
 {
-	while (QTreeWidgetItem *item = searchResultsItem->child(0))
-		delete item;
+	while (QTreeWidgetItem* Item = mSearchResultsItem->child(0))
+		delete Item;
 
-	lcPiecesLibrary* library = lcGetPiecesLibrary();
-	lcArray<PieceInfo*> singleParts, groupedParts;
-
-	library->SearchPieces(searchString.toLocal8Bit().data(), false, singleParts, groupedParts);
+	lcArray<PieceInfo*> singleParts;
+	lcGetPiecesLibrary()->SearchPieces(searchString.toLocal8Bit().data(), singleParts);
 	singleParts.Sort(lcQPartsTreeSortFunc);
 
 	for (int partIndex = 0; partIndex < singleParts.GetSize(); partIndex++)
 	{
 		PieceInfo* partInfo = singleParts[partIndex];
 
-		QTreeWidgetItem* partItem = new QTreeWidgetItem(searchResultsItem, QStringList(partInfo->m_strDescription));
-		partItem->setData(0, PartInfoRole, qVariantFromValue((void*)partInfo));
+		QTreeWidgetItem* partItem = new QTreeWidgetItem(mSearchResultsItem, QStringList(partInfo->m_strDescription));
+		partItem->setData(0, PieceInfoRole, qVariantFromValue((void*)partInfo));
 		partItem->setToolTip(0, QString("%1 (%2)").arg(partInfo->m_strDescription, partInfo->m_strName));
 	}
 
-	setCurrentItem(searchResultsItem);
-	expandItem(searchResultsItem);
-	scrollToItem(searchResultsItem, PositionAtTop);
+	setCurrentItem(mSearchResultsItem);
+	expandItem(mSearchResultsItem);
+	scrollToItem(mSearchResultsItem, PositionAtTop);
 }
 
 void lcQPartsTree::itemExpanded(QTreeWidgetItem *expandedItem)
 {
 	QTreeWidgetItem *parent = expandedItem->parent();
 
-	if (parent || expandedItem == searchResultsItem)
+	if (parent || expandedItem == mModelListItem || expandedItem == mSearchResultsItem)
 		return;
 
 	if (expandedItem->data(0, ExpandedOnceRole).toBool())
@@ -140,7 +164,7 @@ void lcQPartsTree::itemExpanded(QTreeWidgetItem *expandedItem)
 		PieceInfo* partInfo = singleParts[partIndex];
 
 		QTreeWidgetItem* partItem = new QTreeWidgetItem(expandedItem, QStringList(partInfo->m_strDescription));
-		partItem->setData(0, PartInfoRole, qVariantFromValue((void*)partInfo));
+		partItem->setData(0, PieceInfoRole, qVariantFromValue((void*)partInfo));
 		partItem->setToolTip(0, QString("%1 (%2)").arg(partInfo->m_strDescription, partInfo->m_strName));
 
 		if (groupedParts.FindIndex(partInfo) != -1)
@@ -156,13 +180,13 @@ void lcQPartsTree::itemExpanded(QTreeWidgetItem *expandedItem)
 					continue;
 
 				const char* desc = patternedInfo->m_strDescription;
-				int len = strlen(partInfo->m_strDescription);
+				size_t len = strlen(partInfo->m_strDescription);
 
 				if (!strncmp(patternedInfo->m_strDescription, partInfo->m_strDescription, len))
 					desc += len;
 
 				QTreeWidgetItem* patternedItem = new QTreeWidgetItem(partItem, QStringList(desc));
-				patternedItem->setData(0, PartInfoRole, qVariantFromValue((void*)patternedInfo));
+				patternedItem->setData(0, PieceInfoRole, qVariantFromValue((void*)patternedInfo));
 				patternedItem->setToolTip(0, QString("%1 (%2)").arg(patternedInfo->m_strDescription, patternedInfo->m_strName));
 			}
 		}
@@ -193,14 +217,14 @@ void lcQPartsTree::setCurrentPart(PieceInfo *part)
 		strcpy(parentName, part->m_strName);
 		*strchr(parentName, 'P') = '\0';
 
-		PieceInfo *parentPart = library->FindPiece(parentName, false);
+		PieceInfo *parentPart = library->FindPiece(parentName, NULL, false);
 
 		if (parentPart)
 		{
 			for (int itemIndex = 0; itemIndex < categoryItem->childCount(); itemIndex++)
 			{
 				QTreeWidgetItem *item = categoryItem->child(itemIndex);
-				PieceInfo *info = (PieceInfo*)item->data(0, lcQPartsTree::PartInfoRole).value<void*>();
+				PieceInfo *info = (PieceInfo*)item->data(0, PieceInfoRole).value<void*>();
 
 				if (info == parentPart)
 				{
@@ -215,7 +239,7 @@ void lcQPartsTree::setCurrentPart(PieceInfo *part)
 	for (int itemIndex = 0; itemIndex < categoryItem->childCount(); itemIndex++)
 	{
 		QTreeWidgetItem *item = categoryItem->child(itemIndex);
-		PieceInfo *info = (PieceInfo*)item->data(0, lcQPartsTree::PartInfoRole).value<void*>();
+		PieceInfo *info = (PieceInfo*)item->data(0, PieceInfoRole).value<void*>();
 
 		if (info == part)
 		{
@@ -228,7 +252,9 @@ void lcQPartsTree::setCurrentPart(PieceInfo *part)
 
 void lcQPartsTree::startDrag(Qt::DropActions supportedActions)
 {
-	PieceInfo *info = (PieceInfo*)currentItem()->data(0, lcQPartsTree::PartInfoRole).value<void*>();
+	Q_UNUSED(supportedActions);
+
+	PieceInfo *info = (PieceInfo*)currentItem()->data(0, PieceInfoRole).value<void*>();
 
 	if (!info)
 		return;
