@@ -20,6 +20,7 @@ lcQPreferencesDialog::lcQPreferencesDialog(QWidget *parent, void *data) :
 	connect(ui->categoriesTree, SIGNAL(itemSelectionChanged()), this, SLOT(updateParts()));
 	ui->shortcutEdit->installEventFilter(this);
 	connect(ui->commandList, SIGNAL(currentItemChanged(QTreeWidgetItem*, QTreeWidgetItem*)), this, SLOT(commandChanged(QTreeWidgetItem*)));
+	connect(ui->mouseTree, SIGNAL(currentItemChanged(QTreeWidgetItem*, QTreeWidgetItem*)), this, SLOT(MouseTreeItemChanged(QTreeWidgetItem*)));
 
 	options = (lcPreferencesDialogOptions*)data;
 
@@ -30,9 +31,7 @@ lcQPreferencesDialog::lcQPreferencesDialog(QWidget *parent, void *data) :
 	ui->lgeoPath->setText(options->LGEOPath);
 	ui->mouseSensitivity->setValue(options->Preferences.mMouseSensitivity);
 	ui->checkForUpdates->setCurrentIndex(options->CheckForUpdates);
-	ui->centimeterUnits->setChecked((options->Snap & LC_DRAW_CM_UNITS) != 0);
-	ui->noRelativeSnap->setChecked((options->Snap & LC_DRAW_GLOBAL_SNAP) != 0);
-	ui->fixedDirectionKeys->setChecked((options->Snap & LC_DRAW_MOVEAXIS) != 0);
+	ui->fixedDirectionKeys->setChecked((options->Preferences.mFixedAxes) != 0);
 
 	ui->antiAliasing->setChecked(options->AASamples != 1);
 	if (options->AASamples == 8)
@@ -66,9 +65,12 @@ lcQPreferencesDialog::lcQPreferencesDialog(QWidget *parent, void *data) :
 	ui->categoriesTree->setCurrentItem(ui->categoriesTree->topLevelItem(0));
 
 	updateCommandList();
-
 	new lcQTreeWidgetColumnStretcher(ui->commandList, 0);
 	commandChanged(NULL);
+
+	UpdateMouseTree();
+	new lcQTreeWidgetColumnStretcher(ui->mouseTree, 0);
+	MouseTreeItemChanged(NULL);
 }
 
 lcQPreferencesDialog::~lcQPreferencesDialog()
@@ -85,24 +87,14 @@ void lcQPreferencesDialog::accept()
 		return;
 	}
 
-	options->Snap &= ~(LC_DRAW_CM_UNITS | LC_DRAW_GLOBAL_SNAP | LC_DRAW_MOVEAXIS);
-
-	strcpy(options->DefaultAuthor, ui->authorName->text().toLocal8Bit().data());
-	strcpy(options->ProjectsPath, ui->projectsFolder->text().toLocal8Bit().data());
-	strcpy(options->LibraryPath, ui->partsLibrary->text().toLocal8Bit().data());
-	strcpy(options->POVRayPath, ui->povrayExecutable->text().toLocal8Bit().data());
-	strcpy(options->LGEOPath, ui->lgeoPath->text().toLocal8Bit().data());
+	options->DefaultAuthor = ui->authorName->text();
+	options->ProjectsPath = ui->projectsFolder->text();
+	options->LibraryPath = ui->partsLibrary->text();
+	options->POVRayPath = ui->povrayExecutable->text();
+	options->LGEOPath = ui->lgeoPath->text();
 	options->Preferences.mMouseSensitivity = ui->mouseSensitivity->value();
 	options->CheckForUpdates = ui->checkForUpdates->currentIndex();
-
-	if (ui->centimeterUnits->isChecked())
-		options->Snap |= LC_DRAW_CM_UNITS;
-
-	if (ui->noRelativeSnap->isChecked())
-		options->Snap |= LC_DRAW_GLOBAL_SNAP;
-
-	if (ui->fixedDirectionKeys->isChecked())
-		options->Snap |= LC_DRAW_MOVEAXIS;
+	options->Preferences.mFixedAxes = ui->fixedDirectionKeys->isChecked();
 
 	if (!ui->antiAliasing->isChecked())
 		options->AASamples = 1;
@@ -254,7 +246,7 @@ void lcQPreferencesDialog::updateParts()
 	{
 		lcArray<PieceInfo*> singleParts, groupedParts;
 
-		library->SearchPieces(options->Categories[categoryIndex].Keywords, false, singleParts, groupedParts);
+		library->GetCategoryEntries(options->Categories[categoryIndex].Keywords, false, singleParts, groupedParts);
 
 		for (int partIndex = 0; partIndex < singleParts.GetSize(); partIndex++)
 		{
@@ -358,37 +350,31 @@ void lcQPreferencesDialog::on_deleteCategory_clicked()
 
 void lcQPreferencesDialog::on_importCategories_clicked()
 {
-	QString result = QFileDialog::getOpenFileName(this, tr("Import Categories"), "", tr("Text Files (*.txt);;All Files (*.*)"));
+	QString FileName = QFileDialog::getOpenFileName(this, tr("Import Categories"), "", tr("Text Files (*.txt);;All Files (*.*)"));
 
-	if (result.isEmpty())
+	if (FileName.isEmpty())
 		return;
 
-	char fileName[LC_MAXPATH];
-	strcpy(fileName, result.toLocal8Bit().data());
-
-	lcArray<lcLibraryCategory> categories;
-	if (!lcLoadCategories(fileName, categories))
+	lcArray<lcLibraryCategory> Categories;
+	if (!lcLoadCategories(FileName, Categories))
 	{
 		QMessageBox::warning(this, "LeoCAD", tr("Error loading categories file."));
 		return;
 	}
 
-	options->Categories = categories;
+	options->Categories = Categories;
 	options->CategoriesModified = true;
 	options->CategoriesDefault = false;
 }
 
 void lcQPreferencesDialog::on_exportCategories_clicked()
 {
-	QString result = QFileDialog::getSaveFileName(this, tr("Export Categories"), "", tr("Text Files (*.txt);;All Files (*.*)"));
+	QString FileName = QFileDialog::getSaveFileName(this, tr("Export Categories"), "", tr("Text Files (*.txt);;All Files (*.*)"));
 
-	if (result.isEmpty())
+	if (FileName.isEmpty())
 		return;
 
-	char fileName[LC_MAXPATH];
-	strcpy(fileName, result.toLocal8Bit().data());
-
-	if (!lcSaveCategories(fileName, options->Categories))
+	if (!lcSaveCategories(FileName, options->Categories))
 	{
 		QMessageBox::warning(this, "LeoCAD", tr("Error saving categories file."));
 		return;
@@ -410,6 +396,8 @@ void lcQPreferencesDialog::on_resetCategories_clicked()
 
 bool lcQPreferencesDialog::eventFilter(QObject *object, QEvent *event)
 {
+	Q_UNUSED(object);
+
 	if (event->type() == QEvent::KeyPress)
 	{
 		QKeyEvent *keyEvent = static_cast<QKeyEvent*>(event);
@@ -419,7 +407,8 @@ bool lcQPreferencesDialog::eventFilter(QObject *object, QEvent *event)
 			return true;
 
 		Qt::KeyboardModifiers state = keyEvent->modifiers();
-		if (state & Qt::ShiftModifier)
+		QString text = QKeySequence(nextKey).toString();
+		if ((state & Qt::ShiftModifier) && (text.isEmpty() || !text.at(0).isPrint() || text.at(0).isLetter() || text.at(0).isSpace()))
 			nextKey |= Qt::SHIFT;
 		if (state & Qt::ControlModifier)
 			nextKey |= Qt::CTRL;
@@ -435,10 +424,7 @@ bool lcQPreferencesDialog::eventFilter(QObject *object, QEvent *event)
 		return true;
 	}
 
-	if (event->type() == QEvent::Shortcut || event->type() == QEvent::KeyRelease)
-		return true;
-
-	if (event->type() == QEvent::ShortcutOverride)
+	if (event->type() == QEvent::Shortcut || event->type() == QEvent::KeyRelease || event->type() == QEvent::ShortcutOverride)
 	{
 		event->accept();
 		return true;
@@ -503,12 +489,12 @@ void lcQPreferencesDialog::updateCommandList()
 		}
 
 		QTreeWidgetItem *item = new QTreeWidgetItem;
-		QKeySequence sequence(options->KeyboardShortcuts.Shortcuts[actionIdx]);
+		QKeySequence sequence(options->KeyboardShortcuts.mShortcuts[actionIdx]);
 		item->setText(0, subId);
 		item->setText(1, sequence.toString(QKeySequence::NativeText));
 		item->setData(0, Qt::UserRole, qVariantFromValue(actionIdx));
 
-		if (strcmp(options->KeyboardShortcuts.Shortcuts[actionIdx], gCommands[actionIdx].DefaultShortcut))
+		if (options->KeyboardShortcuts.mShortcuts[actionIdx] != gCommands[actionIdx].DefaultShortcut)
 			setShortcutModified(item, true);
 
 		sections[section]->addChild(item);
@@ -536,7 +522,7 @@ void lcQPreferencesDialog::commandChanged(QTreeWidgetItem *current)
 	ui->shortcutGroup->setEnabled(true);
 
 	int shortcutIndex = qvariant_cast<int>(current->data(0, Qt::UserRole));
-	QKeySequence key(options->KeyboardShortcuts.Shortcuts[shortcutIndex]);
+	QKeySequence key(options->KeyboardShortcuts.mShortcuts[shortcutIndex]);
 	ui->shortcutEdit->setText(key.toString(QKeySequence::NativeText));
 }
 
@@ -548,14 +534,14 @@ void lcQPreferencesDialog::on_shortcutAssign_clicked()
 		return;
 
 	int shortcutIndex = qvariant_cast<int>(current->data(0, Qt::UserRole));
-	strcpy(options->KeyboardShortcuts.Shortcuts[shortcutIndex], ui->shortcutEdit->text().toLocal8Bit().data());
+	options->KeyboardShortcuts.mShortcuts[shortcutIndex] = ui->shortcutEdit->text();
 
 	current->setText(1, ui->shortcutEdit->text());
 
-	setShortcutModified(current, strcmp(options->KeyboardShortcuts.Shortcuts[shortcutIndex], gCommands[shortcutIndex].DefaultShortcut) != 0);
+	setShortcutModified(current, options->KeyboardShortcuts.mShortcuts[shortcutIndex] != gCommands[shortcutIndex].DefaultShortcut);
 
-	options->ShortcutsModified = true;
-	options->ShortcutsDefault = false;
+	options->KeyboardShortcutsModified = true;
+	options->KeyboardShortcutsDefault = false;
 }
 
 void lcQPreferencesDialog::on_shortcutRemove_clicked()
@@ -567,38 +553,32 @@ void lcQPreferencesDialog::on_shortcutRemove_clicked()
 
 void lcQPreferencesDialog::on_shortcutsImport_clicked()
 {
-	QString result = QFileDialog::getOpenFileName(this, tr("Import shortcuts"), "", tr("Text Files (*.txt);;All Files (*.*)"));
+	QString FileName = QFileDialog::getOpenFileName(this, tr("Import shortcuts"), "", tr("Text Files (*.txt);;All Files (*.*)"));
 
-	if (result.isEmpty())
+	if (FileName.isEmpty())
 		return;
 
-	char fileName[LC_MAXPATH];
-	strcpy(fileName, result.toLocal8Bit().data());
-
-	lcKeyboardShortcuts shortcuts;
-	if (!lcLoadKeyboardShortcuts(fileName, shortcuts))
+	lcKeyboardShortcuts Shortcuts;
+	if (!Shortcuts.Load(FileName))
 	{
 		QMessageBox::warning(this, "LeoCAD", tr("Error loading keyboard shortcuts file."));
 		return;
 	}
 
-	options->KeyboardShortcuts = shortcuts;
+	options->KeyboardShortcuts = Shortcuts;
 
-	options->ShortcutsModified = true;
-	options->ShortcutsDefault = false;
+	options->KeyboardShortcutsModified = true;
+	options->KeyboardShortcutsDefault = false;
 }
 
 void lcQPreferencesDialog::on_shortcutsExport_clicked()
 {
-	QString result = QFileDialog::getSaveFileName(this, tr("Export shortcuts"), "", tr("Text Files (*.txt);;All Files (*.*)"));
+	QString FileName = QFileDialog::getSaveFileName(this, tr("Export shortcuts"), "", tr("Text Files (*.txt);;All Files (*.*)"));
 
-	if (result.isEmpty())
+	if (FileName.isEmpty())
 		return;
 
-	char fileName[LC_MAXPATH];
-	strcpy(fileName, result.toLocal8Bit().data());
-
-	if (!lcSaveKeyboardShortcuts(fileName, options->KeyboardShortcuts))
+	if (!options->KeyboardShortcuts.Save(FileName))
 	{
 		QMessageBox::warning(this, "LeoCAD", tr("Error saving keyboard shortcuts file."));
 		return;
@@ -610,9 +590,176 @@ void lcQPreferencesDialog::on_shortcutsReset_clicked()
 	if (QMessageBox::question(this, "LeoCAD", tr("Are you sure you want to load the default keyboard shortcuts?"), QMessageBox::Yes | QMessageBox::No) != QMessageBox::Yes)
 		return;
 
-	lcResetKeyboardShortcuts(options->KeyboardShortcuts);
+	options->KeyboardShortcuts.Reset();
 	updateCommandList();
 
-	options->ShortcutsModified = true;
-	options->ShortcutsDefault = true;
+	options->KeyboardShortcutsModified = true;
+	options->KeyboardShortcutsDefault = true;
+}
+
+void lcQPreferencesDialog::UpdateMouseTree()
+{
+	ui->mouseTree->clear();
+
+	for (int ToolIdx = 0; ToolIdx < LC_NUM_TOOLS; ToolIdx++)
+		UpdateMouseTreeItem(ToolIdx);
+}
+
+void lcQPreferencesDialog::UpdateMouseTreeItem(int ItemIndex)
+{
+	Qt::MouseButton Button = options->MouseShortcuts.mShortcuts[ItemIndex].Button;
+	QString Shortcut = QKeySequence(options->MouseShortcuts.mShortcuts[ItemIndex].Modifiers).toString(QKeySequence::NativeText);
+
+	switch (Button)
+	{
+	case Qt::LeftButton:
+		Shortcut += tr("Left Button");
+		break;
+
+	case Qt::MiddleButton:
+		Shortcut += tr("Middle Button");
+		break;
+
+	case Qt::RightButton:
+		Shortcut += tr("Right Button");
+		break;
+
+	default:
+		Shortcut.clear();
+	}
+
+	QTreeWidgetItem* Item = ui->mouseTree->topLevelItem(ItemIndex);
+
+	if (Item)
+		Item->setText(1, Shortcut);
+	else
+		new QTreeWidgetItem(ui->mouseTree, QStringList() << gToolNames[ItemIndex] << Shortcut);
+}
+
+void lcQPreferencesDialog::on_mouseAssign_clicked()
+{
+	QTreeWidgetItem* Current = ui->mouseTree->currentItem();
+
+	if (!Current)
+		return;
+
+	int ButtonIndex = ui->mouseButton->currentIndex();
+	Qt::MouseButton Button = Qt::NoButton;
+	Qt::KeyboardModifiers Modifiers = Qt::NoModifier;
+
+	if (ButtonIndex)
+	{
+		switch (ButtonIndex)
+		{
+		case 1:
+			Button = Qt::LeftButton;
+			break;
+
+		case 2:
+			Button = Qt::MiddleButton;
+			break;
+
+		case 3:
+			Button = Qt::RightButton;
+			break;
+		}
+
+		if (ui->mouseControl->isChecked())
+			Modifiers |= Qt::ControlModifier;
+
+		if (ui->mouseShift->isChecked())
+			Modifiers |= Qt::ShiftModifier;
+
+		if (ui->mouseAlt->isChecked())
+			Modifiers |= Qt::AltModifier;
+
+		for (int ToolIdx = 0; ToolIdx < LC_NUM_TOOLS; ToolIdx++)
+		{
+			if (ToolIdx != ButtonIndex && options->MouseShortcuts.mShortcuts[ToolIdx].Button == Button && options->MouseShortcuts.mShortcuts[ToolIdx].Modifiers == Modifiers)
+			{
+				if (QMessageBox::question(this, tr("Override Shortcut"), tr("This shortcut is already assigned to '%1', do you want to replace it?").arg(gToolNames[ToolIdx]), QMessageBox::Yes | QMessageBox::No) == QMessageBox::No)
+					return;
+
+				options->MouseShortcuts.mShortcuts[ToolIdx].Button = Qt::NoButton;
+				options->MouseShortcuts.mShortcuts[ToolIdx].Modifiers = Qt::NoModifier;
+			}
+		}
+	}
+
+	int ItemIndex = ui->mouseTree->indexOfTopLevelItem(Current);
+	options->MouseShortcuts.mShortcuts[ItemIndex].Button = Button;
+	options->MouseShortcuts.mShortcuts[ItemIndex].Modifiers = Modifiers;
+
+	options->MouseShortcutsModified = true;
+	options->MouseShortcutsDefault = false;
+
+	UpdateMouseTreeItem(ItemIndex);
+}
+
+void lcQPreferencesDialog::on_mouseRemove_clicked()
+{
+	QTreeWidgetItem* Current = ui->mouseTree->currentItem();
+
+	if (!Current)
+		return;
+
+	int ItemIndex = ui->mouseTree->indexOfTopLevelItem(Current);
+	options->MouseShortcuts.mShortcuts[ItemIndex].Button = Qt::NoButton;
+	options->MouseShortcuts.mShortcuts[ItemIndex].Modifiers = Qt::NoModifier;
+
+	options->MouseShortcutsModified = true;
+	options->MouseShortcutsDefault = false;
+
+	UpdateMouseTreeItem(ItemIndex);
+}
+
+void lcQPreferencesDialog::on_mouseReset_clicked()
+{
+	if (QMessageBox::question(this, "LeoCAD", tr("Are you sure you want to load the default mouse shortcuts?"), QMessageBox::Yes | QMessageBox::No) != QMessageBox::Yes)
+		return;
+
+	options->MouseShortcuts.Reset();
+	UpdateMouseTree();
+
+	options->MouseShortcutsModified = true;
+	options->MouseShortcutsDefault = true;
+}
+
+void lcQPreferencesDialog::MouseTreeItemChanged(QTreeWidgetItem* Current)
+{
+	if (!Current)
+	{
+		ui->MouseShortcutGroup->setEnabled(false);
+		return;
+	}
+
+	ui->MouseShortcutGroup->setEnabled(true);
+
+	int ToolIndex = ui->mouseTree->indexOfTopLevelItem(Current);
+
+	Qt::MouseButton Button = options->MouseShortcuts.mShortcuts[ToolIndex].Button;
+
+	switch (Button)
+	{
+	case Qt::LeftButton:
+		ui->mouseButton->setCurrentIndex(1);
+		break;
+
+	case Qt::MiddleButton:
+		ui->mouseButton->setCurrentIndex(2);
+		break;
+
+	case Qt::RightButton:
+		ui->mouseButton->setCurrentIndex(3);
+		break;
+
+	default:
+		ui->mouseButton->setCurrentIndex(0);
+		break;
+	}
+
+	Qt::KeyboardModifiers Modifiers = options->MouseShortcuts.mShortcuts[ToolIndex].Modifiers;
+	ui->mouseControl->setChecked((Modifiers & Qt::ControlModifier) != 0);
+	ui->mouseShift->setChecked((Modifiers & Qt::ShiftModifier) != 0);
+	ui->mouseAlt->setChecked((Modifiers & Qt::AltModifier) != 0);
 }
