@@ -3,12 +3,14 @@
 #include "lc_qcolorpicker.h"
 #include "lc_application.h"
 #include "project.h"
+#include "lc_model.h"
 #include "object.h"
 #include "piece.h"
 #include "camera.h"
 #include "light.h"
 #include "pieceinf.h"
 #include "lc_library.h"
+#include "lc_qutils.h"
 
 // Draw an icon indicating opened/closing branches
 static QIcon drawIndicatorIcon(const QPalette &palette, QStyle *style)
@@ -16,7 +18,6 @@ static QIcon drawIndicatorIcon(const QPalette &palette, QStyle *style)
 	QPixmap pix(14, 14);
 	pix.fill(Qt::transparent);
 	QStyleOption branchOption;
-	QRect r(QPoint(0, 0), pix.size());
 	branchOption.rect = QRect(2, 2, 9, 9); // ### hardcoded in qcommonstyle.cpp
 	branchOption.palette = palette;
 	branchOption.state = QStyle::State_Children;
@@ -69,53 +70,6 @@ static QIcon drawCheckBox(bool value)
 	return QIcon(pixmap);
 }
 
-class lcQPropertiesTreeDelegate : public QItemDelegate
-{
-	Q_OBJECT
-public:
-	lcQPropertiesTreeDelegate(QObject *parent = 0)
-		: QItemDelegate(parent), m_treeWidget(0), m_editedItem(0), m_editedWidget(0), m_disablePainting(false)
-	{}
-
-	void setTreeWidget(lcQPropertiesTree *treeWidget)
-	{
-		m_treeWidget = treeWidget;
-	}
-
-	QWidget *createEditor(QWidget *parent, const QStyleOptionViewItem &option, const QModelIndex &index) const;
-	void updateEditorGeometry(QWidget *editor, const QStyleOptionViewItem &option, const QModelIndex &index) const;
-	void paint(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const;
-	QSize sizeHint(const QStyleOptionViewItem &option, const QModelIndex &index) const;
-	void setModelData(QWidget *, QAbstractItemModel *, const QModelIndex &) const {}
-	void setEditorData(QWidget *, const QModelIndex &) const {}
-	bool eventFilter(QObject *object, QEvent *event);
-
-	QTreeWidgetItem *editedItem() const
-	{
-		return m_editedItem;
-	}
-
-	QWidget *editor() const
-	{
-		return m_editedWidget;
-	}
-
-protected:
-	void drawDecoration(QPainter *painter, const QStyleOptionViewItem &option, const QRect &rect, const QPixmap &pixmap) const;
-	void drawDisplay(QPainter *painter, const QStyleOptionViewItem &option, const QRect &rect, const QString &text) const;
-
-private slots:
-	void slotEditorDestroyed(QObject *object);
-
-private:
-	int indentation(const QModelIndex &index) const;
-
-	lcQPropertiesTree *m_treeWidget;
-	mutable QTreeWidgetItem *m_editedItem;
-	mutable QWidget *m_editedWidget;
-	mutable bool m_disablePainting;
-};
-
 int lcQPropertiesTreeDelegate::indentation(const QModelIndex &index) const
 {
 	if (!m_treeWidget)
@@ -146,6 +100,8 @@ void lcQPropertiesTreeDelegate::slotEditorDestroyed(QObject *object)
 
 QWidget *lcQPropertiesTreeDelegate::createEditor(QWidget *parent, const QStyleOptionViewItem &style, const QModelIndex &index) const
 {
+	Q_UNUSED(style);
+
 	if (index.column() == 1 && m_treeWidget)
 	{
 		QTreeWidgetItem *item = m_treeWidget->indexToItem(index);
@@ -273,8 +229,7 @@ lcQPropertiesTree::lcQPropertiesTree(QWidget *parent) :
 	m_delegate->setTreeWidget(this);
 	setItemDelegate(m_delegate);
 
-	focusObject = NULL;
-	setEmpty();
+	SetEmpty();
 
 	connect(header(), SIGNAL(sectionDoubleClicked(int)), this, SLOT(resizeColumnToContents(int)));
 }
@@ -353,34 +308,92 @@ void lcQPropertiesTree::mousePressEvent(QMouseEvent *event)
 	}
 }
 
-void lcQPropertiesTree::updateFocusObject(Object *newFocusObject)
+void lcQPropertiesTree::Update(const lcArray<lcObject*>& Selection, lcObject* Focus)
 {
-	if (!newFocusObject)
+	lcPropertyWidgetMode Mode = LC_PROPERTY_WIDGET_EMPTY;
+
+	if (Focus)
 	{
-		setEmpty();
-		return;
+		switch (Focus->GetType())
+		{
+		case LC_OBJECT_PIECE:
+			Mode = LC_PROPERTY_WIDGET_PIECE;
+			break;
+
+		case LC_OBJECT_CAMERA:
+			Mode = LC_PROPERTY_WIDGET_CAMERA;
+			break;
+
+		case LC_OBJECT_LIGHT:
+			Mode = LC_PROPERTY_WIDGET_LIGHT;
+			break;
+		}
+	}
+	else
+	{
+		for (int ObjectIdx = 0; ObjectIdx < Selection.GetSize(); ObjectIdx++)
+		{
+			switch (Selection[ObjectIdx]->GetType())
+			{
+			case LC_OBJECT_PIECE:
+				if (Mode == LC_PROPERTY_WIDGET_EMPTY)
+					Mode = LC_PROPERTY_WIDGET_PIECE;
+				else if (Mode != LC_PROPERTY_WIDGET_PIECE)
+				{
+					Mode = LC_PROPERTY_WIDGET_MULTIPLE;
+					ObjectIdx = Selection.GetSize();
+				}
+				break;
+
+			case LC_OBJECT_CAMERA:
+				if (Mode != LC_PROPERTY_WIDGET_EMPTY)
+				{
+					Mode = LC_PROPERTY_WIDGET_MULTIPLE;
+					ObjectIdx = Selection.GetSize();
+				}
+				else
+				{
+					Mode = LC_PROPERTY_WIDGET_CAMERA;
+					Focus = Selection[ObjectIdx];
+				}
+				break;
+
+			case LC_OBJECT_LIGHT:
+				if (Mode != LC_PROPERTY_WIDGET_EMPTY)
+				{
+					Mode = LC_PROPERTY_WIDGET_MULTIPLE;
+					ObjectIdx = Selection.GetSize();
+				}
+				else
+				{
+					Mode = LC_PROPERTY_WIDGET_LIGHT;
+					Focus = Selection[ObjectIdx];
+				}
+				break;
+			}
+		}
 	}
 
-	switch (newFocusObject->GetType())
+	switch (Mode)
 	{
-	case LC_OBJECT_PIECE:
-		setPart(newFocusObject);
+	case LC_PROPERTY_WIDGET_EMPTY:
+		SetEmpty();
 		break;
 
-	case LC_OBJECT_CAMERA:
-		setCamera(newFocusObject);
+	case LC_PROPERTY_WIDGET_PIECE:
+		SetPiece(Selection, Focus);
 		break;
 
-	case LC_OBJECT_CAMERA_TARGET:
-		setCamera(((CameraTarget*)newFocusObject)->GetParent());
+	case LC_PROPERTY_WIDGET_CAMERA:
+		SetCamera(Focus);
 		break;
 
-	case LC_OBJECT_LIGHT:
-		setLight(newFocusObject);
+	case LC_PROPERTY_WIDGET_LIGHT:
+		SetLight(Focus);
 		break;
 
-	case LC_OBJECT_LIGHT_TARGET:
-		setLight(((LightTarget*)newFocusObject)->GetParent());
+	case LC_PROPERTY_WIDGET_MULTIPLE:
+		SetMultiple();
 		break;
 	}
 }
@@ -394,13 +407,25 @@ QWidget *lcQPropertiesTree::createEditor(QWidget *parent, QTreeWidgetItem *item)
 	case PropertyGroup:
 		return NULL;
 
+	case PropertyBool:
+		{
+			QCheckBox *editor = new QCheckBox(parent);
+			bool value = item->data(0, PropertyValueRole).toBool();
+
+			editor->setChecked(value);
+
+			connect(editor, SIGNAL(toggled(bool)), this, SLOT(slotToggled(bool)));
+
+			return editor;
+		}
+
 	case PropertyFloat:
 		{
 			QLineEdit *editor = new QLineEdit(parent);
 			float value = item->data(0, PropertyValueRole).toFloat();
 
 			editor->setValidator(new QDoubleValidator());
-			editor->setText(QString::number(value));
+			editor->setText(lcFormatValue(value));
 
 			connect(editor, SIGNAL(returnPressed()), this, SLOT(slotReturnPressed()));
 
@@ -486,175 +511,208 @@ void lcQPropertiesTree::updateColorEditor(QPushButton *editor, int value) const
 	editor->setText(color->Name);
 }
 
-void lcQPropertiesTree::slotReturnPressed()
+void lcQPropertiesTree::slotToggled(bool Value)
 {
-	if (!focusObject)
-		return;
+	QTreeWidgetItem* Item = m_delegate->editedItem();
+	lcModel* Model = lcGetActiveModel();
 
-	QLineEdit *editor = (QLineEdit*)sender();
-	QTreeWidgetItem *item = m_delegate->editedItem();
-	Project *project = lcGetActiveProject();
-
-	if (focusObject->GetType() == LC_OBJECT_PIECE)
+	if (mWidgetMode == LC_PROPERTY_WIDGET_CAMERA)
 	{
-		Piece *part = (Piece*)focusObject;
+		lcObject* Focus = Model->GetFocusObject();
 
-		if (item == partPositionX || item == partPositionY || item == partPositionZ)
+		if (Focus && Focus->IsCamera())
 		{
-			lcVector3 position = part->mPosition;
-			project->ConvertToUserUnits(position);
-			float value = editor->text().toFloat();
+			lcCamera* Camera = (lcCamera*)Focus;
 
-			if (item == partPositionX)
-				position[0] = value;
-			else if (item == partPositionY)
-				position[1] = value;
-			else if (item == partPositionZ)
-				position[2] = value;
-
-			project->ConvertFromUserUnits(position);
-			project->ModifyObject(focusObject, LC_PART_POSITION, &position);
-		}
-		else if (item == partRotationX || item == partRotationY || item == partRotationZ)
-		{
-			lcVector3 rotation = lcMatrix44ToEulerAngles(part->mModelWorld) * LC_RTOD;
-			float value = editor->text().toFloat();
-
-			if (item == partRotationX)
-				rotation[0] = value;
-			else if (item == partRotationY)
-				rotation[1] = value;
-			else if (item == partRotationZ)
-				rotation[2] = value;
-
-
-			lcVector4 axisAngle = lcMatrix44ToAxisAngle(lcMatrix44FromEulerAngles(rotation * LC_DTOR));
-			axisAngle[3] *= LC_RTOD;
-			project->ModifyObject(focusObject, LC_PART_ROTATION, &axisAngle);
-		}
-		else if (item == partShow)
-		{
-			lcuint32 value = editor->text().toUInt();
-
-			project->ModifyObject(focusObject, LC_PART_SHOW, &value);
-		}
-		else if (item == partHide)
-		{
-			lcuint32 value = editor->text().toUInt();
-
-			project->ModifyObject(focusObject, LC_PART_HIDE, &value);
-		}
-	}
-	else if (focusObject->GetType() == LC_OBJECT_CAMERA)
-	{
-		Camera *camera = (Camera*)focusObject;
-
-		if (item == cameraPositionX || item == cameraPositionY || item == cameraPositionZ)
-		{
-			lcVector3 position = camera->mPosition;
-			project->ConvertToUserUnits(position);
-			float value = editor->text().toFloat();
-
-			if (item == cameraPositionX)
-				position[0] = value;
-			else if (item == cameraPositionY)
-				position[1] = value;
-			else if (item == cameraPositionZ)
-				position[2] = value;
-
-			project->ConvertFromUserUnits(position);
-			project->ModifyObject(focusObject, LC_CAMERA_POSITION, &position);
-		}
-		else if (item == cameraTargetX || item == cameraTargetY || item == cameraTargetZ)
-		{
-			lcVector3 target = camera->mTargetPosition;
-			project->ConvertToUserUnits(target);
-			float value = editor->text().toFloat();
-
-			if (item == cameraTargetX)
-				target[0] = value;
-			else if (item == cameraTargetY)
-				target[1] = value;
-			else if (item == cameraTargetZ)
-				target[2] = value;
-
-			project->ConvertFromUserUnits(target);
-			project->ModifyObject(focusObject, LC_CAMERA_TARGET, &target);
-		}
-		else if (item == cameraUpX || item == cameraUpY || item == cameraUpZ)
-		{
-			lcVector3 up = camera->mUpVector;
-			float value = editor->text().toFloat();
-
-			if (item == cameraUpX)
-				up[0] = value;
-			else if (item == cameraUpY)
-				up[1] = value;
-			else if (item == cameraUpZ)
-				up[2] = value;
-
-			project->ModifyObject(focusObject, LC_CAMERA_UP, &up);
-		}
-		else if (item == cameraFOV)
-		{
-			float value = editor->text().toFloat();
-
-			project->ModifyObject(focusObject, LC_CAMERA_FOV, &value);
-		}
-		else if (item == cameraNear)
-		{
-			float value = editor->text().toFloat();
-
-			project->ModifyObject(focusObject, LC_CAMERA_NEAR, &value);
-		}
-		else if (item == cameraFar)
-		{
-			float value = editor->text().toFloat();
-
-			project->ModifyObject(focusObject, LC_CAMERA_FAR, &value);
-		}
-		else if (item == cameraName)
-		{
-			QString value = editor->text();
-
-			project->ModifyObject(focusObject, LC_CAMERA_NAME, value.toLocal8Bit().data());
+			if (Item == cameraOrtho)
+			{
+				Model->SetCameraOrthographic(Camera, Value);
+			}
 		}
 	}
 }
 
-void lcQPropertiesTree::slotSetValue(int value)
+void lcQPropertiesTree::slotReturnPressed()
 {
-	if (!focusObject)
-		return;
+	QLineEdit* Editor = (QLineEdit*)sender();
+	QTreeWidgetItem* Item = m_delegate->editedItem();
+	lcModel* Model = lcGetActiveModel();
 
-	QTreeWidgetItem *item = m_delegate->editedItem();
-	Project *project = lcGetActiveProject();
-
-	if (focusObject->GetType() == LC_OBJECT_PIECE)
+	if (mWidgetMode == LC_PROPERTY_WIDGET_PIECE)
 	{
-		if (item == partColor)
+		lcPiece* Piece = (mFocus && mFocus->IsPiece()) ? (lcPiece*)mFocus : NULL;
+
+		if (Item == partPositionX || Item == partPositionY || Item == partPositionZ)
 		{
-			project->ModifyObject(focusObject, LC_PART_COLOR, &value);
+			lcVector3 Center;
+			lcMatrix33 RelativeRotation;
+			Model->GetMoveRotateTransform(Center, RelativeRotation);
+			lcVector3 Position = Center;
+			float Value = Editor->text().toFloat();
+
+			if (Item == partPositionX)
+				Position[0] = Value;
+			else if (Item == partPositionY)
+				Position[1] = Value;
+			else if (Item == partPositionZ)
+				Position[2] = Value;
+
+			lcVector3 Distance = Position - Center;
+
+			Model->MoveSelectedObjects(Distance, Distance, true, false, true, true);
+		}
+		else if (Item == partRotationX || Item == partRotationY || Item == partRotationZ)
+		{
+			lcVector3 InitialRotation;
+			if (Piece)
+				InitialRotation = lcMatrix44ToEulerAngles(Piece->mModelWorld) * LC_RTOD;
+			else
+				InitialRotation = lcVector3(0.0f, 0.0f, 0.0f);
+			lcVector3 Rotation = InitialRotation;
+
+			float Value = Editor->text().toFloat();
+
+			if (Item == partRotationX)
+				Rotation[0] = Value;
+			else if (Item == partRotationY)
+				Rotation[1] = Value;
+			else if (Item == partRotationZ)
+				Rotation[2] = Value;
+
+			Model->RotateSelectedPieces(Rotation - InitialRotation, true, false, true, true);
+		}
+		else if (Item == partShow)
+		{
+			lcStep Step = Editor->text().toUInt();
+
+			Model->SetSelectedPiecesStepShow(Step);
+		}
+		else if (Item == partHide)
+		{
+			lcStep Step = Editor->text().toUInt();
+
+			Model->SetSelectedPiecesStepHide(Step);
+		}
+	}
+	else if (mWidgetMode == LC_PROPERTY_WIDGET_CAMERA)
+	{
+		lcCamera* Camera = (mFocus && mFocus->IsCamera()) ? (lcCamera*)mFocus : NULL;
+
+		if (Camera)
+		{
+			if (Item == cameraPositionX || Item == cameraPositionY || Item == cameraPositionZ)
+			{
+				lcVector3 Center = Camera->mPosition;
+				lcVector3 Position = Center;
+				float Value = Editor->text().toFloat();
+
+				if (Item == cameraPositionX)
+					Position[0] = Value;
+				else if (Item == cameraPositionY)
+					Position[1] = Value;
+				else if (Item == cameraPositionZ)
+					Position[2] = Value;
+
+				lcVector3 Distance = Position - Center;
+
+				Model->MoveSelectedObjects(Distance, Distance, true, false, true, true);
+			}
+			else if (Item == cameraTargetX || Item == cameraTargetY || Item == cameraTargetZ)
+			{
+				lcVector3 Center = Camera->mTargetPosition;
+				lcVector3 Position = Center;
+				float Value = Editor->text().toFloat();
+
+				if (Item == cameraTargetX)
+					Position[0] = Value;
+				else if (Item == cameraTargetY)
+					Position[1] = Value;
+				else if (Item == cameraTargetZ)
+					Position[2] = Value;
+
+				lcVector3 Distance = Position - Center;
+
+				Model->MoveSelectedObjects(Distance, Distance, true, false, true, true);
+			}
+			else if (Item == cameraUpX || Item == cameraUpY || Item == cameraUpZ)
+			{
+				lcVector3 Center = Camera->mUpVector;
+				lcVector3 Position = Center;
+				float Value = Editor->text().toFloat();
+
+				if (Item == cameraUpX)
+					Position[0] = Value;
+				else if (Item == cameraUpY)
+					Position[1] = Value;
+				else if (Item == cameraUpZ)
+					Position[2] = Value;
+
+				lcVector3 Distance = Position - Center;
+
+				Model->MoveSelectedObjects(Distance, Distance, true, false, true, true);
+			}
+			else if (Item == cameraFOV)
+			{
+				float Value = Editor->text().toFloat();
+
+				Model->SetCameraFOV(Camera, Value);
+			}
+			else if (Item == cameraNear)
+			{
+				float Value = Editor->text().toFloat();
+
+				Model->SetCameraZNear(Camera, Value);
+			}
+			else if (Item == cameraFar)
+			{
+				float Value = Editor->text().toFloat();
+
+				Model->SetCameraZFar(Camera, Value);
+			}
+			else if (Item == cameraName)
+			{
+				QString Value = Editor->text();
+
+				Model->SetCameraName(Camera, Value.toLocal8Bit().data());
+			}
+		}
+	}
+}
+
+void lcQPropertiesTree::slotSetValue(int Value)
+{
+	QTreeWidgetItem* Item = m_delegate->editedItem();
+	lcModel* Model = lcGetActiveModel();
+
+	if (mWidgetMode == LC_PROPERTY_WIDGET_PIECE)
+	{
+		if (Item == partColor)
+		{
+			Model->SetSelectedPiecesColorIndex(Value);
 
 			QPushButton *editor = (QPushButton*)m_delegate->editor();
-			updateColorEditor(editor, value);
+			updateColorEditor(editor, Value);
 		}
-		else if (item == partID)
+		else if (Item == partID)
 		{
 			QComboBox *editor = (QComboBox*)sender();
 
-			project->ModifyObject(focusObject, LC_PART_ID, editor->itemData(value).value<void*>());
+			Model->SetSelectedPiecesPieceInfo((PieceInfo*)editor->itemData(Value).value<void*>());
 		}
 	}
 }
 
 void lcQPropertiesTree::slotColorButtonClicked()
 {
-	QWidget *parent = (QWidget*)sender();
-	Piece *part = (Piece*)focusObject;
+	int ColorIndex = gDefaultColor;
+	lcObject* Focus = lcGetActiveModel()->GetFocusObject();
+	if (Focus && Focus->IsPiece())
+		ColorIndex = ((lcPiece*)Focus)->mColorIndex;
 
-	lcQColorPickerPopup *popup = new lcQColorPickerPopup(parent, part->mColorIndex);
+	QWidget *parent = (QWidget*)sender();
+	lcQColorPickerPopup *popup = new lcQColorPickerPopup(parent, ColorIndex);
 	connect(popup, SIGNAL(selected(int)), SLOT(slotSetValue(int)));
+	popup->setMinimumSize(300, 200);
 
 	const QRect desktop = QApplication::desktop()->geometry();
 
@@ -664,10 +722,10 @@ void lcQPropertiesTree::slotColorButtonClicked()
 	if (pos.y() < desktop.top())
 		pos.setY(desktop.top());
 
-	if ((pos.x() + popup->sizeHint().width()) > desktop.width())
-		pos.setX(desktop.width() - popup->sizeHint().width());
-	if ((pos.y() + popup->sizeHint().height()) > desktop.bottom())
-		pos.setY(desktop.bottom() - popup->sizeHint().height());
+	if ((pos.x() + popup->width()) > desktop.width())
+		pos.setX(desktop.width() - popup->width());
+	if ((pos.y() + popup->height()) > desktop.bottom())
+		pos.setY(desktop.bottom() - popup->height());
 	popup->move(pos);
 
 	popup->setFocus();
@@ -696,12 +754,10 @@ QTreeWidgetItem *lcQPropertiesTree::addProperty(QTreeWidgetItem *parent, const Q
 	return newItem;
 }
 
-void lcQPropertiesTree::setEmpty()
+void lcQPropertiesTree::SetEmpty()
 {
 	clear();
 
-	focusObject = NULL;
-
 	partPosition = NULL;
 	partPositionX = NULL;
 	partPositionY = NULL;
@@ -730,17 +786,21 @@ void lcQPropertiesTree::setEmpty()
 	cameraUpY = NULL;
 	cameraUpZ = NULL;
 	cameraSettings = NULL;
+	cameraOrtho = NULL;
 	cameraFOV = NULL;
 	cameraNear = NULL;
 	cameraFar = NULL;
 	cameraName = NULL;
+
+	mWidgetMode = LC_PROPERTY_WIDGET_EMPTY;
+	mFocus = NULL;
 }
 
-void lcQPropertiesTree::setPart(Object *newFocusObject)
+void lcQPropertiesTree::SetPiece(const lcArray<lcObject*>& Selection, lcObject* Focus)
 {
-	if (!focusObject || focusObject->GetType() != LC_OBJECT_PIECE)
+	if (mWidgetMode != LC_PROPERTY_WIDGET_PIECE)
 	{
-		setEmpty();
+		SetEmpty();
 
 		partPosition = addProperty(NULL, tr("Position"), PropertyGroup);
 		partPositionX = addProperty(partPosition, tr("X"), PropertyFloat);
@@ -759,42 +819,96 @@ void lcQPropertiesTree::setPart(Object *newFocusObject)
 		partAppearance = addProperty(NULL, tr("Appearance"), PropertyGroup);
 		partColor = addProperty(partAppearance, tr("Color"), PropertyColor);
 		partID = addProperty(partAppearance, tr("Part"), PropertyPart);
+
+		mWidgetMode = LC_PROPERTY_WIDGET_PIECE;
 	}
 
-	focusObject = newFocusObject;
-	Piece *part = (Piece*)focusObject;
+	lcModel* Model = lcGetActiveModel();
+	lcPiece* Piece = (Focus && Focus->IsPiece()) ? (lcPiece*)Focus : NULL;
+	mFocus = Piece;
 
-	lcVector3 position = part->mPosition;
-	lcGetActiveProject()->ConvertToUserUnits(position);
+	lcVector3 Position;
+	lcMatrix33 RelativeRotation;
+	Model->GetMoveRotateTransform(Position, RelativeRotation);
+	partPositionX->setText(1, lcFormatValue(Position[0]));
+	partPositionX->setData(0, PropertyValueRole, Position[0]);
+	partPositionY->setText(1, lcFormatValue(Position[1]));
+	partPositionY->setData(0, PropertyValueRole, Position[1]);
+	partPositionZ->setText(1, lcFormatValue(Position[2]));
+	partPositionZ->setData(0, PropertyValueRole, Position[2]);
 
-	partPositionX->setText(1, QString::number(position[0]));
-	partPositionX->setData(0, PropertyValueRole, position[0]);
-	partPositionY->setText(1, QString::number(position[1]));
-	partPositionY->setData(0, PropertyValueRole, position[1]);
-	partPositionZ->setText(1, QString::number(position[2]));
-	partPositionZ->setData(0, PropertyValueRole, position[2]);
+	lcVector3 Rotation;
+	if (Piece)
+		Rotation = lcMatrix44ToEulerAngles(Piece->mModelWorld) * LC_RTOD;
+	else
+		Rotation = lcVector3(0.0f, 0.0f, 0.0f);
+	partRotationX->setText(1, lcFormatValue(Rotation[0]));
+	partRotationX->setData(0, PropertyValueRole, Rotation[0]);
+	partRotationY->setText(1, lcFormatValue(Rotation[1]));
+	partRotationY->setData(0, PropertyValueRole, Rotation[1]);
+	partRotationZ->setText(1, lcFormatValue(Rotation[2]));
+	partRotationZ->setData(0, PropertyValueRole, Rotation[2]);
 
-	lcVector3 rotation = lcMatrix44ToEulerAngles(part->mModelWorld) * LC_RTOD;
+	lcStep Show = 0;
+	lcStep Hide = 0;
+	int ColorIndex = gDefaultColor;
+	PieceInfo* Info = NULL;
 
-	partRotationX->setText(1, QString::number(rotation[0]));
-	partRotationX->setData(0, PropertyValueRole, rotation[0]);
-	partRotationY->setText(1, QString::number(rotation[1]));
-	partRotationY->setData(0, PropertyValueRole, rotation[1]);
-	partRotationZ->setText(1, QString::number(rotation[2]));
-	partRotationZ->setData(0, PropertyValueRole, rotation[2]);
+	if (Piece)
+	{
+		Show = Piece->GetStepShow();
+		Hide = Piece->GetStepHide();
+		ColorIndex = Piece->mColorIndex;
+		Info = Piece->mPieceInfo;
+	}
+	else
+	{
+		bool FirstPiece = true;
 
-	lcuint32 show = part->GetStepShow();
-	lcuint32 hide = part->GetStepHide();
+		for (int ObjectIdx = 0; ObjectIdx < Selection.GetSize(); ObjectIdx++)
+		{
+			lcObject* Object = Selection[ObjectIdx];
 
-	partShow->setText(1, QString::number(show));
-	partShow->setData(0, PropertyValueRole, show);
-	partHide->setText(1, QString::number(hide));
-	partHide->setData(0, PropertyValueRole, hide);
+			if (Object->GetType() != LC_OBJECT_PIECE)
+				continue;
+
+			lcPiece* SelectedPiece = (lcPiece*)Object;
+
+			if (FirstPiece)
+			{
+				Show = SelectedPiece->GetStepShow();
+				Hide = SelectedPiece->GetStepHide();
+				ColorIndex = SelectedPiece->mColorIndex;
+				Info = SelectedPiece->mPieceInfo;
+
+				FirstPiece = false;
+			}
+			else
+			{
+				if (SelectedPiece->GetStepShow() != Show)
+					Show = 0;
+
+				if (SelectedPiece->GetStepHide() != Hide)
+					Hide = 0;
+
+				if (SelectedPiece->mColorIndex != ColorIndex)
+					ColorIndex = gDefaultColor;
+
+				if (SelectedPiece->mPieceInfo != Info)
+					Info = NULL;
+			}
+		}
+	}
+
+	partShow->setText(1, QString::number(Show));
+	partShow->setData(0, PropertyValueRole, Show);
+	partHide->setText(1, QString::number(Hide));
+	partHide->setData(0, PropertyValueRole, Hide);
 
 	QImage img(16, 16, QImage::Format_ARGB32);
 	img.fill(0);
 
-	lcColor* color = &gColorList[part->mColorIndex];
+	lcColor* color = &gColorList[ColorIndex];
 	QRgb rgb = qRgb(color->Value[0] * 255, color->Value[1] * 255, color->Value[2] * 255);
 	QBrush b(rgb);
 	QPainter painter(&img);
@@ -804,17 +918,19 @@ void lcQPropertiesTree::setPart(Object *newFocusObject)
 
 	partColor->setIcon(1, QIcon(QPixmap::fromImage(img)));
 	partColor->setText(1, color->Name);
-	partColor->setData(0, PropertyValueRole, part->mColorIndex);
+	partColor->setData(0, PropertyValueRole, ColorIndex);
 
-	partID->setText(1, part->mPieceInfo->m_strDescription);
-	partID->setData(0, PropertyValueRole, qVariantFromValue((void*)part->mPieceInfo));
+	partID->setText(1, Info ? Info->m_strDescription : QString());
+	partID->setData(0, PropertyValueRole, qVariantFromValue((void*)Info));
 }
 
-void lcQPropertiesTree::setCamera(Object *newFocusObject)
+void lcQPropertiesTree::SetCamera(lcObject* Focus)
 {
-	if (!focusObject || focusObject->GetType() != LC_OBJECT_CAMERA)
+	Q_UNUSED(Focus);
+
+	if (mWidgetMode != LC_PROPERTY_WIDGET_CAMERA)
 	{
-		setEmpty();
+		SetEmpty();
 
 		cameraPosition = addProperty(NULL, tr("Position"), PropertyGroup);
 		cameraPositionX = addProperty(cameraPosition, tr("X"), PropertyFloat);
@@ -832,64 +948,99 @@ void lcQPropertiesTree::setCamera(Object *newFocusObject)
 		cameraUpZ = addProperty(cameraUp, tr("Z"), PropertyFloat);
 
 		cameraSettings = addProperty(NULL, tr("Up"), PropertyGroup);
+		cameraOrtho = addProperty(cameraSettings, tr("Orthographic"), PropertyBool);
 		cameraFOV = addProperty(cameraSettings, tr("FOV"), PropertyFloat);
 		cameraNear = addProperty(cameraSettings, tr("Near"), PropertyFloat);
 		cameraFar = addProperty(cameraSettings, tr("Far"), PropertyFloat);
 		cameraName = addProperty(cameraSettings, tr("Name"), PropertyString);
+
+		mWidgetMode = LC_PROPERTY_WIDGET_CAMERA;
 	}
 
-	focusObject = newFocusObject;
-	Camera *camera = (Camera*)focusObject;
+	lcCamera* Camera = (Focus && Focus->IsCamera()) ? (lcCamera*)Focus : NULL;
+	mFocus = Camera;
 
-	lcVector3 position = camera->mPosition;
-	lcGetActiveProject()->ConvertToUserUnits(position);
+	lcVector3 Position(0.0f, 0.0f, 0.0f);
+	lcVector3 Target(0.0f, 0.0f, 0.0f);
+	lcVector3 UpVector(0.0f, 0.0f, 0.0f);
+	bool Ortho = false;
+	float FoV = 60.0f;
+	float ZNear = 1.0f;
+	float ZFar = 100.0f;
+	const char* Name = "";
 
-	cameraPositionX->setText(1, QString::number(position[0]));
-	cameraPositionX->setData(0, PropertyValueRole, position[0]);
-	cameraPositionY->setText(1, QString::number(position[1]));
-	cameraPositionY->setData(0, PropertyValueRole, position[1]);
-	cameraPositionZ->setText(1, QString::number(position[2]));
-	cameraPositionZ->setData(0, PropertyValueRole, position[2]);
+	if (Focus)
+	{
+		Position = Camera->mPosition;
+		Target = Camera->mTargetPosition;
+		UpVector = Camera->mUpVector;
 
-	lcVector3 target = camera->mTargetPosition;
-	lcGetActiveProject()->ConvertToUserUnits(target);
+		Ortho = Camera->IsOrtho();
+		FoV = Camera->m_fovy;
+		ZNear = Camera->m_zNear;
+		ZFar = Camera->m_zFar;
+		Name = Camera->GetName();
+	}
 
-	cameraTargetX->setText(1, QString::number(target[0]));
-	cameraTargetX->setData(0, PropertyValueRole, target[0]);
-	cameraTargetY->setText(1, QString::number(target[1]));
-	cameraTargetY->setData(0, PropertyValueRole, target[1]);
-	cameraTargetZ->setText(1, QString::number(target[2]));
-	cameraTargetZ->setData(0, PropertyValueRole, target[2]);
+	cameraPositionX->setText(1, lcFormatValue(Position[0]));
+	cameraPositionX->setData(0, PropertyValueRole, Position[0]);
+	cameraPositionY->setText(1, lcFormatValue(Position[1]));
+	cameraPositionY->setData(0, PropertyValueRole, Position[1]);
+	cameraPositionZ->setText(1, lcFormatValue(Position[2]));
+	cameraPositionZ->setData(0, PropertyValueRole, Position[2]);
 
-	lcVector3 up = camera->mUpVector;
-	lcGetActiveProject()->ConvertToUserUnits(up);
+	cameraTargetX->setText(1, lcFormatValue(Target[0]));
+	cameraTargetX->setData(0, PropertyValueRole, Target[0]);
+	cameraTargetY->setText(1, lcFormatValue(Target[1]));
+	cameraTargetY->setData(0, PropertyValueRole, Target[1]);
+	cameraTargetZ->setText(1, lcFormatValue(Target[2]));
+	cameraTargetZ->setData(0, PropertyValueRole, Target[2]);
 
-	cameraUpX->setText(1, QString::number(up[0]));
-	cameraUpX->setData(0, PropertyValueRole, up[0]);
-	cameraUpY->setText(1, QString::number(up[1]));
-	cameraUpY->setData(0, PropertyValueRole, up[1]);
-	cameraUpZ->setText(1, QString::number(up[2]));
-	cameraUpZ->setData(0, PropertyValueRole, up[2]);
+	cameraUpX->setText(1, lcFormatValue(UpVector[0]));
+	cameraUpX->setData(0, PropertyValueRole, UpVector[0]);
+	cameraUpY->setText(1, lcFormatValue(UpVector[1]));
+	cameraUpY->setData(0, PropertyValueRole, UpVector[1]);
+	cameraUpZ->setText(1, lcFormatValue(UpVector[2]));
+	cameraUpZ->setData(0, PropertyValueRole, UpVector[2]);
 
-	cameraFOV->setText(1, QString::number(camera->m_fovy));
-	cameraFOV->setData(0, PropertyValueRole, camera->m_fovy);
-	cameraNear->setText(1, QString::number(camera->m_zNear));
-	cameraNear->setData(0, PropertyValueRole, camera->m_zNear);
-	cameraFar->setText(1, QString::number(camera->m_zFar));
-	cameraFar->setData(0, PropertyValueRole, camera->m_zFar);
+	cameraOrtho->setText(1, Ortho ? "True" : "False");
+	cameraOrtho->setData(0, PropertyValueRole, Ortho);
+	cameraFOV->setText(1, lcFormatValue(FoV));
+	cameraFOV->setData(0, PropertyValueRole, FoV);
+	cameraNear->setText(1, lcFormatValue(ZNear));
+	cameraNear->setData(0, PropertyValueRole, ZNear);
+	cameraFar->setText(1, lcFormatValue(ZFar));
+	cameraFar->setData(0, PropertyValueRole, ZFar);
 
-	cameraName->setText(1, camera->GetName());
-	cameraName->setData(0, PropertyValueRole, qVariantFromValue((void*)camera->GetName()));
+	cameraName->setText(1, Name);
+	cameraName->setData(0, PropertyValueRole, qVariantFromValue((void*)Name));
 }
 
-void lcQPropertiesTree::setLight(Object *newFocusObject)
+void lcQPropertiesTree::SetLight(lcObject* Focus)
 {
+	Q_UNUSED(Focus);
+
+	SetEmpty();
+	mFocus = NULL;
+
 	// todo: light properties
 }
 
+void lcQPropertiesTree::SetMultiple()
+{
+	if (mWidgetMode != LC_PROPERTY_WIDGET_MULTIPLE)
+	{
+		SetEmpty();
+
+		addProperty(NULL, tr("Multiple Objects Selected"), PropertyGroup);
+
+		mWidgetMode = LC_PROPERTY_WIDGET_MULTIPLE;
+	}
+
+	mFocus = NULL;
+}
+
 bool lcQPropertiesTree::lastColumn(int column) const
 {
 	return header()->visualIndex(column) == columnCount() - 1;
 }
-
-#include "lc_qpropertiestree.moc"
